program lapack_lineqf
  implicit none

  integer, parameter           :: n=1000
  real(kind=8), dimension(n,n) :: A          ! Matrix
  real(kind=8), dimension(n)   :: xref, b    ! Vectors

  !!
  !! Initialization: Generating a random matrix A and a random vector xref.
  !!

  call random_number(A)
  call random_number(xref)

  !!
  !! Perform b := A*xref
  !!

  ! DGEMV (y := alpha*A'*x + beta*y)
  call DGEMV('N',         &
	      n, n,       &  ! number of rows and columns of A
	      1.D0, A, n, &  ! alpha, A, lda
	      xref, 1,    &  ! x, incx
	      0.D0, b, 1)    ! beta, y, incy

  !b = matmul(A,xref);

  !!
  !! Solve A*x = b
  !!
  
  ! LU decomposition with partial pivoting (A = P*L*U)

  ! ...
     
  !!
  !! Finalize
  !!

  ! Test if b == ref
  if (maxval(abs(b-xref)) > 1e-10) then
     STOP 'b != ref'
  end if

end program lapack_lineqf
