#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <math.h>
#include <sys/time.h>
#include <assert.h>

#include <cblas.h>
#include "rng.h"

void rng_init() {
  srand(421);
}

// array_rand initializes an array with random values
void rng_array(double* array, int size) {
  for(int i=0; i<size; i++) {
    array[i] = (double)rand() / (double)(RAND_MAX - 1); // random number in [0.0, 1.0[;
  }
}

int main() {
  int m = 1500, p = 800, n = 2000;
  printf("Multiplying Matrices: C(%dx%d) = A(%dx%d) x B(%dx%d)\n", m, n, m, p, p, n);

  rng_init();
  rng_init2();

  //
  // Matrix Initialization
  //

  double* A  = (double *)malloc(m*p*sizeof(double));
  double* B  = (double *)malloc(p*n*sizeof(double));
  double* C1 = (double *)malloc(m*n*sizeof(double));
  double* C2 = (double *)malloc(m*n*sizeof(double));

  if (A == NULL || B == NULL || C1 == NULL || C2 == NULL) {
    perror("malloc failed");
    free(A); free(B); free(C1); free(C2);
    return EXIT_FAILURE;
  }

  rng_array2(A, m*p);
  rng_array2(B, p*n);
  for (int i = 0; i<m*n; i++) { C1[i] = 0.0; }
  for (int i = 0; i<m*n; i++) { C2[i] = 0.0; }

  //
  // Matrix-Matrix Multiply (reference)
  //

  {
    struct timeval t0, t1; // timer

    printf("J,K,I - ");
   
    gettimeofday(&t0, NULL);

    for (int j = 0; j < n; j++) {
      for (int k = 0; k < p; k++) {
	for (int i = 0; i < m; i++) {
	  C1[i+j*m] += A[i+k*m] * B[k+j*p];       /* col-major */
	  /* C1[i*n+j] += A[i*p+k] * B[k*n+j]; */ /* row-major */
	}
      }
    }

    gettimeofday(&t1, NULL);
    double t = (t1.tv_sec - t0.tv_sec) + (t1.tv_usec - t0.tv_usec) / (double)1000000;
    fprintf(stdout, "Time elapsed: %10.3E sec.\n", t);
  }

  //
  // Matrix-Matrix Multiply (using BLAS)
  //
  {
    struct timeval t0, t1; // timer

    printf("BLAS  - ");
   
    gettimeofday(&t0, NULL);

    // ...

    gettimeofday(&t1, NULL);
    double t = (t1.tv_sec - t0.tv_sec) + (t1.tv_usec - t0.tv_usec) / (double)1000000;
    fprintf(stdout, "Time elapsed: %10.3E sec.\n", t);
  }

  //
  // Finalize
  //

  // Test if C1 == C2
  for (int i = 0; i<m*n; i++) { assert(C2[i] == C1[i]); }

  free(A);
  free(B);
  free(C1);
  free(C2);
  rng_free2();

  return EXIT_SUCCESS;
}
