program matmatmul
  use rng_wrapper

  implicit none

  integer, parameter           :: m=1500, p=800, n=2000
  real(kind=8), dimension(m,p) :: A           ! matrices
  real(kind=8), dimension(p,n) :: B           !
  real(kind=8), dimension(m,n) :: C1          !
  real(kind=8), dimension(m,n) :: C2          !
  integer                      :: i, j, k
  integer                      :: t1, t2, ir  ! timer
  real(kind=8)                 :: t           ! timer
  real(kind=8)                 :: alpha, beta ! BLAS input argument

  call rng_init2()

  print '("Multiplying Matrices: C(", I5, "x", I5, ")", &
       & "= A(", I5, "x", I5, ") x B(", I5, "x", I5, ")")', &
       & m, n, m, p, p, n

  !!
  !! Matrix Initialization
  !!

  !call random_number(A)
  !call random_number(B)
  call rng_array2(A, m*p)
  call rng_array2(B, p*n)

  C1 = 0.0
  C2 = 0.0

  !!
  !! Matrix-Matrix Multiply (reference)
  !!

  write (*,"(A)",advance="no"), "J,K,I - "

  ! Temps elapsed de reference.
  call system_clock(count=t1, count_rate=ir)

  ! Produit de matrices
  do j = 1, n
     do k = 1, p
        do i = 1, m
           C1(i,j) = C1(i,j) + A(i,k) * B(k,j)
        end do
     end do
  end do

  ! alternative: Fortran matmul
  ! C1 = matmul(A, B)  

  ! Temps elapsed final
  call system_clock(count=t2, count_rate=ir)
  t = real(t2 - t1, kind=8)/real(ir, kind=8)
  print '("Time elapsed : ", 1PE10.3, " sec.")', t
 
  !!
  !! Matrix-Matrix Multiply (using BLAS)
  !!

  write (*,"(A)",advance="no"), "BLAS  - "

  ! Temps elapsed de reference.
  call system_clock(count=t1, count_rate=ir)

  alpha = 1.0 
  beta  = 0.0
  call DGEMM('N', 'N', m, n, p, alpha, A, m, B, p, beta, C2, m)

  ! Temps elapsed final
  call system_clock(count=t2, count_rate=ir)
  t = real(t2 - t1, kind=8)/real(ir, kind=8)
  print '("Time elapsed : ", 1PE10.3, " sec.")', t

  !!
  !! Finalize
  !!

  ! Test if C1 == C2
  if (maxval(abs(C2-C1)) > 0) then
     STOP 'C1 != C2'
  end if

  call rng_free2()

end program matmatmul
